@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2020
+/* Copyright (c) 1997-2024
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -100,7 +100,7 @@ BigObject polarize(BigObject p_in, OptionSet options)
       if (facets_property_read) {
          orthogonalize(entire(rows(lin)));
          project_to_orthogonal_complement(pts, lin); // cheap if lin==0
-         p_out.take( facets_property == "FACETS" ? Str("RAYS") : Str("INPUT_RAYS") ) << pts;
+         p_out.take( facets_property == "FACETS" ? Str("RAYS") : Str("INPUT_RAYS") ) << remove_zero_rows(pts);
       }
 
       // if p was obtained by some transformation,
