@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2024
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -25,8 +26,8 @@ namespace polymake { namespace polytope {
 template<typename Scalar>
 Matrix<Scalar>
 prism_coord(const Matrix<Scalar>& V,
-            int& n_vertices, int& n_vertices_out,
-            const Set<int>& rays,
+            Int& n_vertices, Int& n_vertices_out,
+            const Set<Int>& rays,
             const Scalar& z, const Scalar& z_prime)
 {
 
@@ -42,29 +43,27 @@ prism_coord(const Matrix<Scalar>& V,
 }
 
 template<typename Scalar>
-perl::Object prism(perl::Object p_in, const Scalar& z, const Scalar& z_prime, perl::OptionSet options)
+BigObject prism(BigObject p_in, const Scalar& z, const Scalar& z_prime, OptionSet options)
 {
+   if (z == z_prime)
+      throw std::runtime_error("prism: z and z' must be different");
+   if (options["group"] && !p_in.exists("GROUP"))
+      throw std::runtime_error("prism: group of the base polytope needs to be provided in order to compute group of the pyramid.");
 
-   if(z==z_prime)
-        throw std::runtime_error("prism: z and z' must be different");
-   if(options["group"] && !p_in.exists("GROUP"))
-         throw std::runtime_error("prism: group of the base polytope needs to be provided in order to compute group of the pyramid.");
-
-
-   int n_vertices=0, n_vertices_out=0;
-   Set<int> rays;
+   Int n_vertices = 0, n_vertices_out = 0;
+   Set<Int> rays;
    if (!options["no_coordinates"])
       p_in.give("FAR_FACE") >> rays;
 
-   perl::Object p_out(perl::ObjectType::construct<Scalar>("Polytope"));
+   BigObject p_out("Polytope", mlist<Scalar>());
    p_out.set_description() << "prism over " << p_in.name() << endl;
 
    if (options["no_coordinates"] || p_in.exists("VERTICES_IN_FACETS")) {
-      const IncidenceMatrix<> VIF=p_in.give("VERTICES_IN_FACETS");
-      n_vertices=VIF.cols();
-      n_vertices_out=2*n_vertices-rays.size();
+      const IncidenceMatrix<> VIF = p_in.give("VERTICES_IN_FACETS");
+      n_vertices = VIF.cols();
+      n_vertices_out = 2*n_vertices-rays.size();
 
-      Set<int> far_facet=accumulate(cols(VIF.minor(All,rays)), operations::mul());
+      Set<Int> far_facet = accumulate(cols(VIF.minor(All,rays)), operations::mul());
       if (far_facet.size() > 1) // if size()>1 then FAR_FACE is not a facet
          far_facet.clear();
 
@@ -83,7 +82,7 @@ perl::Object prism(perl::Object p_in, const Scalar& z, const Scalar& z_prime, pe
 
    if (options["no_coordinates"]) {
       if (p_in.exists("COMBINATORIAL_DIM")) {
-         const int dim=p_in.give("COMBINATORIAL_DIM");
+         const Int dim = p_in.give("COMBINATORIAL_DIM");
          p_out.take("COMBINATORIAL_DIM") << dim+1;
       }
 
@@ -97,37 +96,32 @@ perl::Object prism(perl::Object p_in, const Scalar& z, const Scalar& z_prime, pe
       p_out.take("VERTICES") << V_out;
    }
 
-   if(options["group"]){
-      Array< Array< int > > gens = p_in.give("GROUP.VERTICES_ACTION.GENERATORS");
+   if (options["group"]) {
+      Array<Array<Int>> gens = p_in.give("GROUP.VERTICES_ACTION.GENERATORS");
 
-      for(auto i = entire(gens); !i.at_end(); ++i){
+      for (auto i = entire(gens); !i.at_end(); ++i) {
           (*i).append(n_vertices,entire(*i));
-          for(int j = n_vertices; j<n_vertices_out; ++j)
-            (*i)[j]+=n_vertices;
+          for (Int j = n_vertices; j < n_vertices_out; ++j)
+            (*i)[j] += n_vertices;
       }
 
-      Array<int> swap(sequence(n_vertices,n_vertices));
+      Array<Int> swap(sequence(n_vertices, n_vertices));
       swap.append(n_vertices,entire(sequence(0,n_vertices)));
       gens.resize(gens.size()+1,swap);
 
-      perl::Object a("group::PermutationAction");
-      a.take("GENERATORS") << gens;
-
-      perl::Object g("group::Group");
+      BigObject a("group::PermutationAction", "GENERATORS", gens);
+      BigObject g("group::Group", "canonicalGroup");
       g.set_description() << "canonical group induced by the group of the base polytope" << endl;
-      g.set_name("canonicalGroup");
       p_out.take("GROUP") << g;
       p_out.take("GROUP.VERTICES_ACTION") << a;
    }
 
-
-
    if (!options["no_labels"]) {
       std::vector<std::string> labels(n_vertices_out);
-      common::read_labels(p_in, "VERTEX_LABELS", non_const(select(labels, sequence(0,n_vertices))));
+      common::read_labels(p_in, "VERTEX_LABELS", select(labels, sequence(0, n_vertices)));
       const std::string tick="'";
       copy_range(entire(attach_operation(select(labels, sequence(0,n_vertices)-rays),
-                                         constant(tick), operations::add())),
+                                         same_value(tick), operations::add())),
                  labels.begin()+n_vertices);
       p_out.take("VERTEX_LABELS") << labels;
    }
@@ -152,8 +146,15 @@ UserFunctionTemplate4perl("# @category  Producing a polytope from polytopes"
                           "# @example The following saves the prism over the square and the interval [-2,2] to the"
                           "# variable $p, and then prints a nice representation of its vertices."
                           "# > $p = prism(cube(2),-2);"
-                          "# > print labeled($p->VERTICES,$p->VERTEX_LABELS);"
-                          "# | 0:1 -1 -1 -2 1:1 1 -1 -2 2:1 -1 1 -2 3:1 1 1 -2 0':1 -1 -1 2 1':1 1 -1 2 2':1 -1 1 2 3':1 1 1 2",
+                          "# > print rows_labeled($p->VERTICES,$p->VERTEX_LABELS);"
+                          "# | 0:1 -1 -1 -2"
+                          "# | 1:1 1 -1 -2"
+                          "# | 2:1 -1 1 -2"
+                          "# | 3:1 1 1 -2"
+                          "# | 0':1 -1 -1 2"
+                          "# | 1':1 1 -1 2"
+                          "# | 2':1 -1 1 2"
+                          "# | 3':1 1 1 2",
                           "prism<Scalar>(Polytope<type_upgrade<Scalar>>; type_upgrade<Scalar>=-1, type_upgrade<Scalar>=-$_[1], {group => 0, no_coordinates => undef, no_labels => 0})");
 } }
 
