@@ -1,6 +1,7 @@
-#  Copyright (c) 1997-2018
-#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-#  http://www.polymake.org
+#  Copyright (c) 1997-2024
+#  Ewgenij Gawrilow, Michael Joswig, and the polymake team
+#  Technische Universität Berlin, Germany
+#  https://polymake.org
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -20,34 +21,40 @@ use warnings qw(FATAL void syntax misc);
 package Polymake;
 
 # an identifier (alphanumeric, first character not a digit)
-declare $id_re=qr{(?> (?!\d)\w+ )}x;
+declare $id_re = qr{(?> (?!\d)\w+ )}x;
+
+# a property name (alphanumeric, first character is a letter)
+declare $prop_name_re = qr{(?> [a-zA-Z]\w* )}x;
 
 # a list of identifiers (separated by commas)
-declare $ids_re=qr{(?> $id_re (?: \s*,\s* $id_re)* )}xo;
+declare $ids_re = qr{(?> $id_re (?: \s*,\s* $id_re)* )}xo;
 
-# an identifier with optional attributes
-declare $id_attrs_re=qr{(?> $id_re (?: \( \s* $ids_re \s* \) )? )}xo;
+# a property name with optional attributes
+declare $prop_name_attrs_re = qr{(?> $prop_name_re (?: \( \s* $ids_re \s* \) )? )}xo;
 
 # a hierarchical identifier (identifiers connected with dots)
-declare $hier_id_re=qr{(?> $id_re (?: \. $id_re)* )}xo;
+declare $hier_id_re = qr{(?> $id_re (?: \. $id_re)* )}xo;
+
+# property path, property names connected with dots
+declare $prop_path_re = qr{(?> $prop_name_re (?: \. $prop_name_re)* )}xo;
 
 # a list of hierarchical identifiers (separated by commas)
-declare $hier_ids_re=qr{(?> $hier_id_re (?: \s*,\s* $hier_id_re)* )}xo;
+declare $hier_ids_re = qr{(?> $hier_id_re (?: \s*,\s* $hier_id_re)* )}xo;
 
-# a hierarchical identifier with trailing alternatives (separated by bars)
-declare $hier_id_alt_re=qr{(?> $hier_id_re (?: \s*\|\s* $id_re)* )}xo;
+# list of alternative property paths
+declare $prop_path_alt_re = qr{(?> $prop_path_re (?: \s*\|\s* $prop_path_re)* )}xo;
 
-# an element of a rule header: hierarchical identifier with optional attributes
-declare $hier_id_attrs_re=qr{(?> $id_attrs_re (?: \. $id_attrs_re)* )}xo;
+# property path with optional attributes
+declare $prop_path_attrs_re = qr{(?> $prop_name_attrs_re (?: \. $prop_name_attrs_re)* )}xo;
 
 # a lone identifier
-declare $id_only_re=qr{ ($id_re) \s*$ }xo;
+declare $id_only_re = qr{ ($id_re) \s*$ }xo;
 
 # fully qualified name
-declare $qual_id_re=qr{(?> $id_re (?: :: $id_re)* )}xo;
+declare $qual_id_re = qr{(?> $id_re (?: :: $id_re)* )}xo;
 
 # a list of fully qualified names
-declare $qual_ids_re=qr{(?> $qual_id_re (?: \s*,\s* $qual_id_re)* )}xo;
+declare $qual_ids_re = qr{(?> $qual_id_re (?: \s*,\s* $qual_id_re)* )}xo;
 
 # unqualified name
 declare $unqual_id_re=qr{(?<!::) (?<!\w) $id_re (?! :: )}xo;
@@ -64,11 +71,15 @@ declare $non_quote_re=qr{ [^"'] }x;
 # anything but a quote character or white space
 declare $non_quote_space_re=qr{ [^"'\s] }x;
 
-# a sequence of symbols not being delimiters (paired brackets or quotes), escaped delimiters, or arrows
-my $non_delims=qr{(?: [^()\[\]{}<>'"\#]++
-                    | (?<=\\)(?<!\\\\) [()\[\]{}<>"'\#]
-                    | (?<=[-=])(?<!<=) >
-                   )*+ }x;
+# a sequence of characters not being delimiters (paired brackets or quotes), escaped delimiters, or arrows
+my $non_delim_chars = qr( [^()\[\]{}<>'"\#]++
+                          | (?<=\\)(?<!\\\\) [()\[\]{}<>"'\#]
+                          | (?<=[-=])(?<!<=) > )x;
+
+my $non_delim = qr{(?: $non_delim_chars )*+ }xo;
+
+# same in a multi-line string, including comments
+my $non_delim_multi = qr{(?: $non_delim_chars | (?m:\#.*\n) )*+ }xo;
 
 declare $single_quoted_re=qr{(?: [^']+ | (?<=\\)(?<!\\\\) ' )*+}x;
 declare $double_quoted_re=qr{(?: [^"]+ | (?<=\\)(?<!\\\\) " )*+}x;
@@ -82,32 +93,35 @@ declare $quoted_re=qr{(?: ' (?'quoted' $single_quoted_re) ' |
 
 # an expression in parentheses, braces, brackets, or quotes
 # 1 capturing group
-declare $confined_re=qr{ ( \( $non_delims (?: (?-1) $non_delims )* \) |
-                           \[ $non_delims (?: (?-1) $non_delims )* \] |
-                           \{ $non_delims (?: (?-1) $non_delims )* \} |
-                            < $non_delims (?: (?-1) $non_delims )*  > |
-                            $anon_quoted_re                            ) }xo;
+declare $confined_re = qr{ ( \( $non_delim (?: (?-1) $non_delim )* \) |
+                             \[ $non_delim (?: (?-1) $non_delim )* \] |
+                             \{ $non_delim (?: (?-1) $non_delim )* \} |
+                              < $non_delim (?: (?-1) $non_delim )*  > |
+                              $anon_quoted_re ) }xo;
+
+# same in a multi-line string, including comments
+declare $confined_multiline_re = qr{ ( \( $non_delim_multi (?: (?-1) $non_delim_multi )* \) |
+                                       \[ $non_delim_multi (?: (?-1) $non_delim_multi )* \] |
+                                       \{ $non_delim_multi (?: (?-1) $non_delim_multi )* \} |
+                                        < $non_delim_multi (?: (?-1) $non_delim_multi )*  > |
+                                        $anon_quoted_re ) }xo;
 
 # a piece of code with proper nested embraced and quoted subexpressions
 # 1 capturing group
-declare $balanced_re=qr{ $non_delims (?: $confined_re $non_delims )* }xo;
+declare $balanced_re=qr{ $non_delim (?: $confined_re $non_delim )* }xo;
 
 # as above, but allowing some unmatched open braces
 # used in TAB completion
-declare $open_balanced_re=qr{ (?: $non_delims (?: \( (?: $balanced_re \) )?+ |
-                                                  \[ (?: $balanced_re \] )?+ |
-                                                  \{ (?: $balanced_re \} )?+ |
-                                                   < (?: $balanced_re  > )?+ |
-                                                   $anon_quoted_re           |
-                                                   $ ) )* }xo;
+declare $open_balanced_re=qr{ (?: $non_delim (?: \( (?: $balanced_re \) )?+ |
+                                                 \[ (?: $balanced_re \] )?+ |
+                                                 \{ (?: $balanced_re \} )?+ |
+                                                  < (?: $balanced_re  > )?+ |
+                                                  $anon_quoted_re           |
+                                                  $ ) )* }xo;
 
 # a piece of code with all quoted strings properly closed
 declare $quote_balanced_re=qr{ $non_quote_re* (?: $anon_quoted_re $non_quote_re* )* }xo;
 
-# an expression in a list; < > are treated as comparison ops, therefore not trying to match them pairwise
-# 1 capturing group
-declare $expression_re=qr{ (?: (?! <) $confined_re | [^,'"()\[\]{}]*+ )+ }xo;
-
 # parameter list in angle brackets, recursively referring to the outer group
 # this is a fragment of a regex, referring to a group outside this string, therefore not defined as qr{}
 my $param_list='(?: \s*<\s* (?-1) (?: \s*,\s* (?-1) )* \s*> )?+';
@@ -137,22 +151,32 @@ declare $type_exprs_re=qr{ ($type_expr_re (?: \s*,\s* (?-5))*+) }xo;
 # 6 capturing groups
 declare $type_param_re=qr{ (?'name' $id_re) (?: \s*=\s* (?'default' $type_expr_re) )?+ }xo;
 
-# a list of type parameters, enclosed in angle brackets, with optional typecheck expression
-# 10 capturing groups
-declare $type_params_re=qr{ \s*<\s* (?'tparams' ($type_param_re (?: \s*,\s* (?-7) )*+ )) \s*>
-                            (?: \s*\[ (?'typecheck' $balanced_re) \] )?+ }xo;
+# list of type parameters in angle brackets with optional default values
+# 7 capturing groups
+declare $type_params_re=qr{ \s*<\s* (?'tparams' ($type_param_re (?: \s*,\s* (?-7) )*+ )) \s*> }xo;
 
-# beginning of a declaration of a type or function, optionally parameterized
-declare $paramed_decl_re=qr{ (?'lead_name' $id_re) (?: $type_params_re )?+ }xo;
+# list of type parameters in angle brackets with optional default values
+# 7 capturing groups
+declare $type_params_variadic_re=qr{ \s*<\s* (?'tparams' ($type_param_re (?: \s*,\s* (?-7) )*+ ))
+                                             (?('default') | (?: \s* (?'ellipsis' \.\.\.))? ) \s*> }xo;
+
+# type-checking expressions
+declare $typechecks_re=qr{ \s*\[ (?'typecheck' $balanced_re) \] }xo;
+
+# beginning of a declaration of a type or function
+declare $parametrized_decl_re=qr{ (?'lead_name' $id_re) (?: $type_params_re (?: $typechecks_re )?+ )?+ }xo;
 
 # function declaration with optional signature
 declare $sub_re=qr{ (?'name' $id_re) \b (?: \s*\( (?'signature' $balanced_re) \) )? }xo;
 
 # function declaration with optional template parameters and signature
-declare $paramed_sub_re=qr{ $paramed_decl_re (?: \s*\( (?'signature' $balanced_re) \) )?+ (?= [\s:;\{]) }xo;
+declare $parametrized_sub_re=qr{ $parametrized_decl_re (?: \s*\( (?'signature' $balanced_re) \) )?+ (?= [\s:;\{]) }xo;
 
 # overloaded function declaration with optional labels
-declare $labeled_sub_re=qr{ (?:(?'labels' $hier_ids_re) \s*:\s*)? $paramed_sub_re }xo;
+declare $labeled_sub_re=qr{ (?:(?'label' $hier_id_re) \s*:\s*)? $parametrized_sub_re }xo;
+
+# attributes of an argument in a function signature, maybe empty
+declare $func_arg_attrs_re=qr{(?=\s*[:&]) (?> (?: \s*:\s* $id_re )* (?: \s* &&? (?: \s* const\b)?)? )}xo;
 
 # rule input clause
 declare $rule_input_re=qr{ $hier_id_re (?: \s*[|,]\s* $hier_id_re )* }xo;
@@ -166,26 +190,22 @@ declare $directory_re=qr{ ^(.*) / [^/]+$ }x;
 # directory part of a command, expecting trailing arguments
 declare $directory_of_cmd_re=qr{ ^(.*) / [^/]+ (?: $ | \s)}x;
 
-# an empty line
-declare $empty_line_re=qr{^ [ \t]* \n}xm;
-
-# an empty line or separator (a line of hashes)
-declare $empty_or_separator_line=qr{^ (?: [ \t]* | \#{2,} ) \n}xm;
-
 # an empty line with possible comments
 declare $nonsignificant_line_re=qr{^ [ \t]* (?:\#.*)? $}xm;
 
-# a line with some contents (like perl code)
-declare $significant_line_re=qr{^ [ \t]* (?! $ | \#) }xm;
-
 # beginning of a complete statement
 declare $statement_start_re=qr{(?: ^ | [;\}] )\s*}x;
 
 # beginning of a function argument list
 declare $args_start_re=qr{(?'args_start' \s+ | \s*\(\s* )}x;
 
-# beginning of a method argument list
-declare $method_args_start_re=qr{(?'args_start' \s*\(\s* )}x;
+# an expression in a list, like an argument in a function call
+# This pattern tries hard to recognize commas within type expressions,
+# while all other < and > signs being interpreted as comparsion operators
+declare $expression_re=qr{ (?: (?! <) $confined_re | (?> \$$qual_id_re | $type_re ) | [^,'"()\[\]{}]*+ )+ }xo;
+
+# variable sigil (scalar, array or hash)
+declare $var_sigil_re=qr{(?: (?'scalar'\$)|[\@%] )}xo;
 
 # end of a source file
 declare $end_of_source_file="__END__";
